@@ -22,26 +22,27 @@ import struct
 from checkbox.contrib.bpickle import dumps, loads
 from checkbox.lib.selector import Selector, SelectorIO
 
-class FifoBase(object):
 
-    mode = None
+class FifoBase:
+
+    flags = None
 
     def __init__(self, path, timeout=None):
         self.path = path
-        self.file = open(path, self.mode)
+        self.fileno = os.open(path, self.flags)
         self._timeout = timeout
 
     def __del__(self):
         self.close()
 
     def close(self):
-        self.file.close()
+        os.close(self.fileno)
 
     def wait_for(self, operation):
         if self._timeout is not None:
             selector = Selector()
             selector.set_timeout(self._timeout)
-            selector.add_fd(self.file.fileno(), operation)
+            selector.add_fd(self.fileno, operation)
 
             selector.execute()
 
@@ -49,57 +50,53 @@ class FifoBase(object):
                 return False
         return True
 
+
 class FifoReader(FifoBase):
 
-    #on Linux, opening a FIFO in read-write mode is non-blocking and
-    #succeeds even if other end is not open as per FIFO(7)
-    mode = "w+"
+    flags = os.O_RDWR
 
-    def read_string(self):
+    def read_bytes(self):
         # Check if a connection arrived within the timeout
         if not self.wait_for(SelectorIO.READ):
             return None
 
         size = struct.calcsize("i")
-        length_string = self.file.read(size)
-        if not length_string:
-            return ""
+        length_bytes = os.read(self.fileno, size)
+        if not length_bytes:
+            return b""
 
-        length = struct.unpack(">i", length_string)[0]
-        return self.file.read(length)
+        length = struct.unpack(">i", length_bytes)[0]
+        return os.read(self.fileno, length)
 
     def read_object(self):
-        string = self.read_string()
-        if not string:
+        _bytes = self.read_bytes()
+        if not _bytes:
             return None
 
-        return loads(string)
+        return loads(_bytes)
 
 
 class FifoWriter(FifoBase):
 
-    #on Linux, opening a FIFO in read-write mode is non-blocking and
-    #succeeds even if other end is not open as per FIFO(7)
-    mode = "w+"
+    flags = os.O_RDWR | os.O_CREAT | os.O_TRUNC
 
-    def write_string(self, string):
+    def write_bytes(self, _bytes):
 
-        # Wait until I can write 
+        # Wait until I can write
         if not self.wait_for(SelectorIO.WRITE):
-            return None 
+            return None
 
-        length = len(string)
-        length_string = struct.pack(">i", length)
-        self.file.write(length_string)
-        self.file.write(string)
-        self.file.flush()
+        length = len(_bytes)
+        length_bytes = struct.pack(">i", length)
+        os.write(self.fileno, length_bytes)
+        os.write(self.fileno, _bytes)
         return length
 
     def write_object(self, object):
-        string = dumps(object)
-        return self.write_string(string)
+        _bytes = dumps(object)
+        return self.write_bytes(_bytes)
 
 
-def create_fifo(path, mode=0666):
+def create_fifo(path, mode=0o666):
     os.mkfifo(path, mode)
     return path
