@@ -18,12 +18,12 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox.impl.box
-=================
+:mod:`plainbox.impl.box` -- command line interface
+==================================================
 
-Internal implementation of plainbox
+.. warning::
 
- * THIS MODULE DOES NOT HAVE STABLE PUBLIC API *
+    THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
 
 from argparse import ArgumentParser
@@ -34,6 +34,7 @@ from logging import basicConfig
 from logging import getLogger
 from os.path import join
 import argparse
+import re
 import sys
 
 from plainbox import __version__ as version
@@ -71,10 +72,16 @@ class CheckBoxCommandMixIn:
         group.add_argument(
             '-i', '--include-pattern', action="append",
             metavar='PATTERN', default=[], dest='include_pattern_list',
-            help="Run jobs matching the given pattern")
+            help=("Run jobs matching the given regular expression. Matches "
+                  "from the start to the end of the line."))
+        group.add_argument(
+            '-x', '--exclude-pattern', action="append",
+            metavar="PATTERN", default=[], dest='exclude_pattern_list',
+            help=("Do not run jobs matching the given regular expression. "
+                  "Matches from the start to the end of the line."))
         # TODO: Find a way to handle the encoding of the file
         group.add_argument(
-            '-W', '--whitelist',
+            '-w', '--whitelist',
             metavar="WHITELIST",
             type=FileType("rt"),
             help="Load whitelist containing run patterns")
@@ -88,18 +95,36 @@ class CheckBoxCommandMixIn:
     def _get_matching_job_list(self, ns, job_list):
         # Find jobs that matched patterns
         matching_job_list = []
+        # Pre-seed the include pattern list with data read from
+        # the whitelist file.
         if ns.whitelist:
-            ns.include_pattern_list.extend([pattern.strip() for pattern in
-                                        ns.whitelist.readlines()])
+            ns.include_pattern_list.extend([
+                pattern.strip()
+                for pattern in ns.whitelist.readlines()])
+        # Decide which of the known jobs to include
         for job in job_list:
-            for pattern in ns.include_pattern_list:
-                if fnmatch(job.name, pattern):
-                    matching_job_list.append(job)
+            # Reject all jobs that match any of the exclude
+            # patterns, matching strictly from the start to
+            # the end of the line.
+            for pattern in ns.exclude_pattern_list:
+                if re.match(r"^{pattern}$".format(pattern=pattern), job.name):
                     break
+            else:
+                # Then accept (include) all job that matches
+                # any of include patterns, matching strictly
+                # from the start to the end of the line.
+                for pattern in ns.include_pattern_list:
+                    if re.match(r"^{pattern}$".format(pattern=pattern),
+                               job.name):
+                        matching_job_list.append(job)
+                        break
         return matching_job_list
 
 
 class SpecialCommand(PlainBoxCommand, CheckBoxCommandMixIn):
+    """
+    Implementation of ``$ plainbox special``
+    """
 
     def invoked(self, ns):
         job_list = self.get_job_list(ns)
@@ -261,6 +286,17 @@ class RunCommand(PlainBoxCommand, CheckBoxCommandMixIn):
             raise SystemExit(str(exc))
         return exporter
 
+    def ask_for_resume(self, prompt=None, allowed=None):
+        # FIXME: Add support/callbacks for a GUI
+        if prompt is None:
+            prompt = "Do you want to resume the previous session [Y/n]? "
+        if allowed is None:
+            allowed = ('', 'y', 'Y', 'n', 'N')
+        answer = None
+        while answer not in allowed:
+            answer = input(prompt)
+        return False if answer in ('n', 'N') else True
+
     def _run_jobs(self, ns, job_list, exporter):
         # Compute the run list, this can give us notification about problems in
         # the selected jobs. Currently we just display each problem
@@ -268,8 +304,13 @@ class RunCommand(PlainBoxCommand, CheckBoxCommandMixIn):
         print("[ Analyzing Jobs ]".center(80, '='))
         # Create a session that handles most of the stuff needed to run jobs
         session = SessionState(job_list)
-        self._update_desired_job_list(session, matching_job_list)
         with session.open():
+            if session.previous_session_file():
+                if self.ask_for_resume():
+                    session.resume()
+                else:
+                    session.clean()
+            self._update_desired_job_list(session, matching_job_list)
             if (sys.stdin.isatty() and sys.stdout.isatty() and not
                     ns.not_interactive):
                 outcome_callback = self.ask_for_outcome
