@@ -24,6 +24,7 @@ plainbox.impl.exporter.test_init
 Test definitions for plainbox.impl.exporter module
 """
 
+from tempfile import TemporaryDirectory
 from unittest import TestCase
 
 from plainbox.impl.exporter import SessionStateExporterBase
@@ -31,7 +32,7 @@ from plainbox.impl.exporter import classproperty
 from plainbox.impl.session import SessionState
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.result import JobResult, IOLogRecord
-from plainbox.impl.testing_utils import make_job, make_job_result
+from plainbox.impl.testing_utils import make_io_log, make_job, make_job_result
 
 
 class ClassPropertyTests(TestCase):
@@ -95,7 +96,7 @@ class SessionStateExporterBaseTests(TestCase):
         }
         self.assertEqual(data, expected_data)
 
-    def make_realistic_test_session(self):
+    def make_realistic_test_session(self, session_dir):
         # Create a more realistic session with two jobs but with richer set
         # of data in the actual jobs and results.
         job_a = JobDefinition({
@@ -115,17 +116,17 @@ class SessionStateExporterBaseTests(TestCase):
             'job': job_a,
             'outcome': 'pass',
             'return_code': 0,
-            'io_log': (
-                IOLogRecord(0, 'stdout', b'testing\n'),
-            )
+            'io_log': make_io_log(
+                (IOLogRecord(0, 'stdout', b'testing\n'),),
+                session_dir)
         })
         result_b = JobResult({
             'job': job_b,
             'outcome': 'pass',
             'return_code': 0,
-            'io_log': (
-                IOLogRecord(0, 'stdout', b'ready: yes\n'),
-            )
+            'io_log': make_io_log(
+                (IOLogRecord(0, 'stdout', b'ready: yes\n'),),
+                session_dir)
         })
         session.update_job_result(job_a, result_a)
         session.update_job_result(job_b, result_b)
@@ -139,10 +140,11 @@ class SessionStateExporterBaseTests(TestCase):
         #   - OPTION_FLATTEN_IO_LOG
         # The implementation favours SQUASH_IO_LOG
         # and thus the code below tests that option
-        exporter = self.TestSessionStateExporter(
-            self.TestSessionStateExporter.supported_option_list)
-        session = self.make_realistic_test_session()
-        data = exporter.get_session_data_subset(session)
+        with TemporaryDirectory() as scratch_dir:
+            exporter = self.TestSessionStateExporter(
+                self.TestSessionStateExporter.supported_option_list)
+            session = self.make_realistic_test_session(scratch_dir)
+            data = exporter.get_session_data_subset(session)
         expected_data = {
             'job_list': ['job_a', 'job_b'],
             'run_list': ['job_b', 'job_a'],
@@ -166,6 +168,8 @@ class SessionStateExporterBaseTests(TestCase):
                     'command': 'echo ready: yes',
                     'io_log': ['cmVhZHk6IHllcwo='],
                 }
+            },
+            'attachment_map': {
             }
         }
         # This is just to make debugging easier
