@@ -18,16 +18,20 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox.impl.integration_tests
-===============================
+:mod:`plainbox.impl.integration_tests` -- integration tests
+===========================================================
 
-Integration tests for checkbox scripts
+.. warning::
+
+    THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
 
 from tempfile import TemporaryDirectory
 from unittest import TestCase
 import json
 import os
+import shutil
+import tempfile
 
 from pkg_resources import resource_filename, resource_isdir, resource_listdir
 
@@ -41,6 +45,14 @@ class IntegrationTests(TestCaseWithParameters):
 
     parameter_names = ('job_name',)
 
+    def setUp(self):
+        # session data are kept in XDG_CACHE_HOME/plainbox/.session
+        # To avoid resuming a real session, we have to select a temporary
+        # location instead
+        self._sandbox = tempfile.mkdtemp()
+        self._env = os.environ
+        os.environ['XDG_CACHE_HOME'] = self._sandbox
+
     @classmethod
     def _gen_job_name_values(cls, package='plainbox', root='data/'):
         """
@@ -94,3 +106,7 @@ class IntegrationTests(TestCaseWithParameters):
             expected_result = json.load(stream)
         # Check that results match expected values
         self.assertEqual(actual_result, expected_result)
+
+    def tearDown(self):
+        shutil.rmtree(self._sandbox)
+        os.environ = self._env
