@@ -19,10 +19,11 @@
 import os
 
 from gettext import gettext as _
+from string import printable
 
 from checkbox.lib.safe import safe_make_directory
 
-from checkbox.properties import Path
+from checkbox.properties import Path, String
 from checkbox.plugin import Plugin
 from checkbox.reports.launchpad_report import LaunchpadReportManager
 
@@ -32,6 +33,9 @@ class LaunchpadReport(Plugin):
     # Filename where submission information is cached.
     filename = Path(default="%(checkbox_data)s/submission.xml")
 
+    # Prompt for place to save the submission file
+    submission_path_prompt = String(default="")
+
     # XML Schema
     schema = Path(default="%(checkbox_share)s/report/hardware-1_0.rng")
 
@@ -62,12 +66,24 @@ class LaunchpadReport(Plugin):
              ("report-package", self.report_package),
              ("report-uname", self.report_uname),
              ("report-system_id", self.report_system_id),
+             ("report-suites", self.report_suites),
+             ("report-review", self.report_review),             
              ("report-tests", self.report_tests)]:
             self._manager.reactor.call_on(rt, rh)
 
         # Launchpad report should be generated last.
         self._manager.reactor.call_on("report", self.report, 100)
 
+        #Ask where to put submission file
+        self._manager.reactor.call_on("prompt-begin", self.prompt_begin, 110)
+
+    def prompt_begin(self, interface):
+        if self.submission_path_prompt:
+            new_filename = interface.show_entry(
+                self.submission_path_prompt, self.filename)
+            if new_filename != "":
+                self.filename = new_filename
+
     def report_attachments(self, attachments):
         for attachment in attachments:
             name = attachment["name"]
@@ -80,7 +96,7 @@ class LaunchpadReport(Plugin):
             elif "udev_attachment" in name:
                 self._report["hardware"]["udev"] = attachment["data"]
 
-            else:
+            elif all(c in printable for c in attachment["data"]):
                 self._report["context"].append({
                     "command": attachment["command"],
                     "data": attachment["data"]})
@@ -122,6 +138,7 @@ class LaunchpadReport(Plugin):
         self._report["summary"]["system_id"] = system_id
 
     def report_tests(self, tests):
+        self.tests = tests
         for test in tests:
             question = {
                 "name": test["name"],
@@ -137,10 +154,6 @@ class LaunchpadReport(Plugin):
         report_manager = LaunchpadReportManager(
             "system", "1.0", stylesheet_path, self.schema)
         payload = report_manager.dumps(self._report).toprettyxml("")
-        if not report_manager.validate(payload):
-            self._manager.reactor.fire("report-error", _("""\
-The generated report seems to have validation errors,
-so it might not be processed by Launchpad."""))
 
         # Write the report
         stylesheet_data = open(self.stylesheet).read() % os.environ
@@ -149,7 +162,62 @@ so it might not be processed by Launchpad."""))
         safe_make_directory(directory)
         open(self.filename, "w").write(payload)
 
+        # Validate the report
+        if not report_manager.validate(payload):
+            self._manager.reactor.fire("report-error", _("""\
+The generated report seems to have validation errors,
+so it might not be processed by Launchpad."""))
+
         self._manager.reactor.fire("launchpad-report", self.filename)
 
+    def report_review(self, interface):
+        """
+        Show test report in the interface
+        """
+        report = {}
+
+        def add_job(job):
+            is_suite = 'type' in job and job['type'] == 'suite'
+            if 'suite' in job:
+                suite_name = job['suite']
+                parent_node = add_job(self.suites[suite_name])
+
+                if is_suite:
+                    if job['description'] in parent_node:
+                        return parent_node[job['description']]
+
+                    node = {}
+                    parent_node[job['description']] = node
+                    return node
+                parent_node[job['name']] = job
+            else:
+                if is_suite:
+                    field = 'description'
+                else:
+                    field = 'name'
+
+                if job[field] in report:
+                    return report[job[field]]
+
+                node = {}
+                report[job[field]] = node
+                return node
+
+        for test in self.tests:
+            add_job(test)
+
+        try:
+            interface.show_report("Test case results report", report)
+        except NotImplementedError:
+            # Silently ignore the interfaces that don't implement the method
+            pass
+
+    def report_suites(self, suites):
+        """
+        Get tests results and store it
+        to display them later
+        """
+        self.suites = dict([(suite['name'], suite) for suite in suites])
+
 
 factory = LaunchpadReport
