@@ -25,13 +25,20 @@ from checkbox.job import UNINITIATED
 from checkbox.lib.resolver import Resolver
 
 from checkbox.plugin import Plugin
-from checkbox.user_interface import PREV
+from checkbox.properties import String
+from checkbox.user_interface import CONTINUE_ANSWER, RERUN_ANSWER
 
 from gettext import gettext as _
 
 
 class SuitesPrompt(Plugin):
 
+    deselect_warning = String(default=_("""\
+Unselecting a test will invalidate your submission for Ubuntu Friendly. \
+If you plan to participate in Ubuntu Friendly, please, select all tests. \
+You can always skip individual tests if you don't have the needed equipment.\
+"""))
+
     @property
     def persist(self):
         if self._persist is None:
@@ -46,12 +53,13 @@ class SuitesPrompt(Plugin):
         self._jobs = {}
         self._statuses = {}
         self._persist = None
-        self._recover = False 
+        self._recover = False
 
         for (rt, rh) in [
              ("begin-persist", self.begin_persist),
              ("begin-recover", self.begin_recover),
-             ("report-suite", self.report_suite)]:
+             ("report-suite", self.report_suite),
+             ("store-access", self.store_access)]:
             self._manager.reactor.call_on(rt, rh)
 
         for (rt, rh) in [
@@ -64,7 +72,8 @@ class SuitesPrompt(Plugin):
         self._persist = persist
 
     def begin_recover(self, recover):
-        self._recover = recover
+        if recover in [CONTINUE_ANSWER, RERUN_ANSWER]:
+            self._recover = True
 
         if not self._recover:
             self.persist.remove("default")
@@ -72,6 +81,9 @@ class SuitesPrompt(Plugin):
     def report_suite(self, suite):
         suite.setdefault("type", "suite")
 
+    def store_access(self, store):
+        self.store = store
+
     def report_job(self, job):
         if job.get("type") == "suite":
             attribute = "description"
@@ -87,19 +99,38 @@ class SuitesPrompt(Plugin):
 
     def prompt_gather(self, interface):
         # Resolve dependencies
+        interface.show_progress_start(_("Gathering information from your system..."))
         resolver = Resolver()
-        for key in self._jobs.iterkeys():
+        for key in self._jobs.keys():
             depends = self._depends.get(key, [])
             resolver.add(key, *depends)
 
         # Build options
         options = {}
+        self._manager.reactor.fire("expose-msgstore")
+        offset = self.store.get_pending_offset()
+        self.store.set_pending_offset(0)
+        messages = self.store.get_pending_messages()
+        self.store.add_pending_offset(offset)
+        tests = dict([(m["name"], m) for m in messages
+              if m.get("type") in ("test", "metric")])
+
+        def walk_dependencies(job, all_dependencies):
+            for dependency in resolver.get_dependencies(job)[:-1]:
+                walk_dependencies(dependency, all_dependencies)
+            all_dependencies.append(job)
+
         for job in resolver.get_dependents():
             suboptions = options
-            dependencies = resolver.get_dependencies(job)
+            dependencies = []
+            walk_dependencies(job, dependencies)
             for dependency in dependencies:
-                value = self._statuses.get(dependency, {})
-                suboptions = suboptions.setdefault(self._jobs[dependency], value)
+                if dependency in tests:
+                    value = tests[dependency]["status"]
+                else:
+                    value = self._statuses.get(dependency, {})
+                suboptions = suboptions.setdefault(self._jobs[dependency],
+                                                   value)
 
         # Build defaults
         defaults = self.persist.get("default")
@@ -107,19 +138,22 @@ class SuitesPrompt(Plugin):
             defaults = copy.deepcopy(options)
 
         # Get results
-        defaults = interface.show_tree(_("Select the suites to test"),
-            options, defaults)
+        interface.show_progress_stop()
+        defaults = interface.show_tree(
+            _("Choose tests to run on your system:"),
+            options, defaults, self.deselect_warning)
         self.persist.set("default", defaults)
 
         # Get tests to ignore
         def get_ignore_jobs(options, results):
             jobs = []
-            if isinstance(options,dict):
-                for k, v in options.iteritems():
+            if isinstance(options, dict):
+                for k, v in options.items():
                     if v == UNINITIATED and k not in results:
                         jobs.append(k)
                     else:
-                        jobs.extend(get_ignore_jobs(options[k], results.get(k, {})))
+                        jobs.extend(get_ignore_jobs(options[k],
+                                                    results.get(k, {})))
 
             return jobs
 
